package geometrie;

import java.io.*;
import java.math.*;

/**
 * classe Vecteur
 * classe reprsentant un vecteur dans l'espace
 */
 
public class Vecteur {
    
    private double x,y,z;   //les coordonnes du vecteur
    
   /**
    * constructeur Vecteur 
    * construit un Vecteur avec 0 comme coordonnes par dfaut
    */
    public Vecteur() {
        x=0; y=0; z=0;
    }
    
   /**
    * constructeur Vecteur 
    * construit un Vecteur avec les coordonnes spcifies
    * @param x_ la coordonne x
    * @param y_ la coordonne y
    * @param z_ la coordonne z
    */
    public Vecteur(double x_, double y_, double z_) {
        x=x_; y=y_; z=z_;
    }
    
   /**
    * constructeur Vecteur 
    * construit un Vecteur norm avec les coordonnes angulaires spcifies
    * @param theta l'angle theta
    * @param phi   l'angle phi
    */
    public Vecteur(double theta, double phi) {
        set_euler(theta, phi);
    }
    
   /**
    * constructeur Vecteur 
    * construit un Vecteur norm avec les coordonnes angulaires spcifies
    * @param ang les angles dfinissant le vecteur
    */
    public Vecteur(Angles ang) {
        set_euler(ang.get_theta(), ang.get_phi());
    }
    
   /**
    * methode rpt
    * retourne un angle  partir des valeurs de son sinus et cosinus
    * @param cs la valeur du cosinus
    * @param sn la valeur du sinus
    * @return l'angle correspondant aux sinus et cosinus donns
    */
    private double rpt(double cs, double sn) {
        if (cs>1 && cs<1+1e-10) cs=1;
        if (cs<-1 && cs>-1-1e-10) cs=-1;
        if (sn>1 && sn<1+1e-10) sn=1;
        if (sn<-1 && sn>-1-1e-10) sn=-1;

        double angle=Math.acos(cs);
        if (sn<0) angle=-angle;
        return angle;
    }
    
   /**
    * methode norme
    * retourne la norme du vecteur
    * @return la norme du vecteur
    */
    public double norme() {return Math.sqrt(x*x+y*y+z*z);}

   /**
    * methode norme_carre
    * retourne la norme au carr du vecteur
    * @return la norme au carr du vecteur
    */
    public double norme_carre() {return x*x+y*y+z*z;}
    
   /**
    * methode get_x
    * retourne la valeur de la coordonne x
    * @return la valeur de la coordonne x
    */
    public double get_x() {return x;}

   /**
    * methode get_y
    * retourne la valeur de la coordonne y
    * @return la valeur de la coordonne y
    */
    public double get_y() {return y;}

   /**
    * methode get_z
    * retourne la valeur de la coordonne z
    * @return la valeur de la coordonne z
    */
    public double get_z() {return z;}
    
   /**
    * methode get_theta
    * retourne la valeur de l'angle theta form par le vecteur
    * @return la valeur de l'angle theta form par le vecteur
    */
    public double get_theta() {
        double r=norme();
        return rpt(z/r, Math.sqrt(x*x+y*y)/r);
    }

   /**
    * methode get_phi
    * retourne la valeur de l'angle phi form par le vecteur
    * @return la valeur de l'angle phi form par le vecteur
    */
    public double get_phi() {
        double theta=get_theta();
        double r=Math.sqrt(x*x+y*y);
        if (Math.abs(Math.sin(theta))<1e-10) return 0; //phi indtermin
        return rpt(x/r, y/r);
    }
    
   /**
    * methode get_theta_phi
    * retourne la valeur des angles theta et phi forms par le vecteur
    * @return la valeur des angles theta et phi forms par le vecteur
    */
    public Angles get_theta_phi() {
        double r=norme();
        double phi, theta = rpt(z/r, Math.sqrt(x*x+y*y)/r);

        r=Math.sqrt(x*x+y*y);
        if (Math.abs(Math.sin(theta))<1e-10) phi=0; //phi indtermin
        phi = rpt(x/r, y/r);
        return new Angles(theta, phi);
    }

   /**
    * methode set_x
    * permet de modifier la coordonne x
    * @param v la valeur de la coordonne x
    */
    public void set_x(double v) {x=v;}

   /**
    * methode set_y
    * permet de modifier la coordonne y
    * @param v la valeur de la coordonne y
    */
    public void set_y(double v) {y=v;}

   /**
    * methode set_z
    * permet de modifier la coordonne z
    * @param v la valeur de la coordonne z
    */
    public void set_z(double v) {z=v;}

   /**
    * methode set
    * permet de modifier les coordonnes du vecteur
    * @param x_ la valeur de la coordonne x
    * @param y_ la valeur de la coordonne y
    * @param z_ la valeur de la coordonne z
    */
    public void set(double x_, double y_, double z_) {
        x=x_; y=y_; z=z_;
    }

   /**
    * methode set_euler
    * permet de modifier les coordonnes angulaires du vecteur norm
    * @param theta la valeur de l'angle theta
    * @param phi   la valeur de l'angle phi
    */
    public void set_euler(double theta, double phi) {
        x = Math.sin(theta) * Math.cos(phi);
        y = Math.sin(theta) * Math.sin(phi);
        z = Math.cos(theta);
    }

   /**
    * methode set_euler
    * permet de modifier les coordonnes angulaires du vecteur norm
    * @param ang les angles dfinissant le vecteur
    */
    public void set_euler(Angles ang) {
        set_euler(ang.get_theta(), ang.get_phi());
    }
    
   /**
    * methode scalaire
    * retourne le produit scalaire du vecteur avec un autre
    * @param v second vecteur
    * @return le produit scalaire du vecteur avec un autre
    */
    public double scalaire(Vecteur v) {
        return x*v.get_x() + y*v.get_y() + z*v.get_z();
    }
    
   /**
    * methode vectoriel
    * retourne le produit vectoriel du vecteur avec un autre
    * @param v second vecteur
    * @return le produit vectoriel du vecteur avec un autre
    */
    public Vecteur vectoriel(Vecteur v) {
        return new Vecteur(y*v.get_z()-z*v.get_y(), -x*v.get_z()+z*v.get_x(), x*v.get_y()-y*v.get_x());
    }
    
   /**
    * methode normalise
    * normalise le vecteur
    */
    public void normalise() {
        double n=norme();
        x/=n;
        y/=n;
        z/=n;
    }
    
   /**
    * methode 
    * retourne le vecteur normalis
    * @return le vecteur normalis
    */
    public Vecteur normalised() {
        double n=norme();
        return new Vecteur(x/n, y/n, z/n);
    }
    
   /**
    * methode fois
    * retourne le vecteur multipli par un scalaire
    * @param d le facteur multiplicatif  affecter au vecteur
    * @return le vecteur multipli par un scalaire
    */
    public Vecteur fois(double d) {
        return new Vecteur(x*d, y*d, z*d);
    }
    
   /**
    * methode plus
    * retourne la somme du vecteur avec un autre
    * @param v second vecteur
    * @return la somme du vecteur avec un autre
    */
    public Vecteur plus(Vecteur v) {
        return new Vecteur(x+v.x, y+v.y, z+v.z);
    }

   /**
    * methode rotate_x
    * retourne le vecteur aprs rotation d'un angle donn autour de l'axe x
    * @param angle l'angle de rotation
    * @return le vecteur aprs rotation
    */
    public Vecteur rotate_x(double angle) {
        return new Vecteur(x, y*Math.cos(angle)-z*Math.sin(angle), z*Math.cos(angle)+y*Math.sin(angle));
    }

   /**
    * methode rotate_y
    * retourne le vecteur aprs rotation d'un angle donn autour de l'axe y
    * @param angle l'angle de rotation
    * @return le vecteur aprs rotation
    */
    public Vecteur rotate_y(double angle) {
        return new Vecteur(x*Math.cos(angle)+z*Math.sin(angle), y, z*Math.cos(angle)-x*Math.sin(angle));
    }

   /**
    * methode rotate_z
    * retourne le vecteur aprs rotation d'un angle donn autour de l'axe z
    * @param angle l'angle de rotation
    * @return le vecteur aprs rotation
    */
    public Vecteur rotate_z(double angle) {
        return new Vecteur(x*Math.cos(angle)-y*Math.sin(angle), y*Math.cos(angle)+x*Math.sin(angle), z);
    }

   /**
    * methode rotated_x
    * tourne le vecteur d'un angle donn autour de l'axe x
    * @param angle l'angle de rotation
    */
    public void rotated_x(double angle) {
        double Y = y;
        y=Y*Math.cos(angle)-z*Math.sin(angle);
        z=z*Math.cos(angle)+Y*Math.sin(angle);
    }

   /**
    * methode rotated_y
    * tourne le vecteur d'un angle donn autour de l'axe y
    * @param angle l'angle de rotation
    */
    public void rotated_y(double angle) {
        double Z = z;
        z=Z*Math.cos(angle)-x*Math.sin(angle);
        x=x*Math.cos(angle)+Z*Math.sin(angle);
    }

   /**
    * methode rotated_z
    * tourne le vecteur d'un angle donn autour de l'axe z
    * @param angle l'angle de rotation
    */
    public void rotated_z(double angle) {
        double X = x;
        x=X*Math.cos(angle)-y*Math.sin(angle);
        y=y*Math.cos(angle)+X*Math.sin(angle);
    }

   /**
    * methode oppose
    * change le signe des coordonnes (opration v  ->  -v)
    */
    public void oppose() {
        x=-x;
        y=-y;
        z=-z;
    }

   /**
    * methode affiche
    * affiche les coordonnes
    */
    public void affiche() {
        System.out.println(x+" "+y+" "+z);
    }

   /**
    * methode print
    * enregistre les coordonnes de deux points matrialisant le vecteur dans le flux spcifi
    * @param out flux de sortie
    * @param P   le point o commencer la reprsentation du vecteur
    * @param d   la longueur de la matrialisation du vecteur
    */
    public void print(PrintWriter out, Point P, double d) {
        P.print(out);
        P.translate(this.fois(d)).print(out);
        out.println();
        out.println();
    }

}
